#include <dos.h>
#include "dpmi.h"

#ifndef	HIWORD
#define	HIWORD(x)	(x >> 16)
#endif

#ifndef	LOWORD
#define	LOWORD(x)	(x & 0xFFFF)
#endif

#ifndef	NULLF
#define	NULLF		0l
#endif

unsigned DPMIError;

//---------------------------------------------------------------
// Allocate a selector in the LDT
//---------------------------------------------------------------

unsigned AllocLDTDesc(void)
{
  union REGS regs;
  regs.x.ax = 0x0000;
  regs.x.cx = 1;
  DPMIError = int86(0x31, &regs, &regs);
  return (regs.x.cflag) ? 0 : regs.x.ax;
}

//---------------------------------------------------------------
// Free a selector in the LDT
//---------------------------------------------------------------

int FreeLDTDesc(unsigned sel)
{
  union REGS regs;
  regs.x.ax = 0x0001;
  regs.x.bx = sel;
  DPMIError = int86(0x31, &regs, &regs);
  return !regs.x.cflag;
}

//---------------------------------------------------------------
// Set the base of a selector
//---------------------------------------------------------------

int SetSegBase(unsigned sel, unsigned long seg)
{
  union REGS regs;
  regs.x.ax = 7;
  regs.x.bx = sel;
  regs.x.cx = HIWORD(seg);
  regs.x.dx = LOWORD(seg);
  DPMIError = int86(0x31, &regs, &regs);
  return !regs.x.cflag;
}

//---------------------------------------------------------------
// Set the limit of a selector
//---------------------------------------------------------------

int SetSegLimit(unsigned sel, unsigned long limit)
{
  union REGS regs;
  regs.x.ax = 8;
  regs.x.bx = sel;
  regs.x.cx = HIWORD(limit);
  regs.x.dx = LOWORD(limit);
  DPMIError = int86(0x31, &regs, &regs);
  return !regs.x.cflag;
}

//---------------------------------------------------------------
// Create a protected mode alias for a real mode segment
//---------------------------------------------------------------

void far *AllocDOSAlias(unsigned seg, unsigned ofs)
{
  unsigned sel;
  if ((sel = AllocLDTDesc()) != 0)
  {
    if (SetSegBase(sel, seg << 4) && SetSegLimit(sel, 0x10000l))
      return MK_FP(sel, ofs);
    FreeLDTDesc(sel); 
  }
  return NULLF;
}

//---------------------------------------------------------------
// Free an alias created bye CreateDOSAlias
//---------------------------------------------------------------

int FreeDOSAlias(void far *alias)
{
  return FreeLDTDesc(FP_SEG(alias));
}

//---------------------------------------------------------------
// simulate a real mode interrupt
//---------------------------------------------------------------
// intno = interrupt #
// stack = size of stack needed (in words)
// dregs -> cpu registers structure
 
int SimulateRealModeInt(int intno, int stackwords, DPMIREGS *dregs)
{
  union REGS uregs;
  struct SREGS sregs;
  segread(&sregs);
  uregs.x.ax = 0x0300;
  uregs.h.bl = intno;
  uregs.x.cx = stackwords;
  uregs.x.di = FP_OFF(dregs);
  sregs.es = FP_SEG(dregs);
  DPMIError = int86x(0x31, &uregs, &uregs, &sregs);
  return !uregs.x.cflag;
}
